import pytest
from unittest.mock import Mock, patch

from arelle.ModelObject import ModelObject
from arelle.ModelValue import qname
from arelle.XmlUtil import (
    escapedNode,
    collapseWhitespace,
    replaceWhitespace,
)
from arelle.XhtmlValidate import (
    htmlEltUriAttrs,
    resolveHtmlUri,
)


@patch('arelle.XmlUtil.htmlEltUriAttrs', new=htmlEltUriAttrs)
@patch('arelle.XmlUtil.resolveHtmlUri', new=resolveHtmlUri)
def test_opaque_uris_not_path_normed():
    uri = 'data:image/png;base64,iVBORw0K//a'
    elt_attrs = {'src': uri}
    elt = Mock(
        spec=ModelObject,

        localName='img',
        namespaceURI='http://www.w3.org/1999/xhtml',
        modelDocument=Mock(htmlBase=None),
        prefix=None,

        get=elt_attrs.get,
        items=elt_attrs.items,
    )
    elt.qname = qname(elt)
    node = escapedNode(elt, start=True, empty=True, ixEscape=True, ixResolveUris=True)
    assert node == f'<img src="{uri}">'


REPLACE_WHITESPACE_TESTS = [
    ("\n", " "),
    ("\r", " "),
    ("\t", " "),
    ("\r\t\n", " " * 3),
    ("\t\t \n\n \r\r", " " * 8),
    ("\r \n \t", " " * 5),
    # literal or entity \v and \f are illegal in XML but we shouldn't be touching them
    ("\r\v\t", " \v "),
    ("\r\f\t", " \f "),
    # Python's whitespace (\s, str.isspace) definition includes em space (U+2003)
    # but XSD replace does not so we shouldn't be touching it
    ("1\u2003", "1\u2003"),
    (" m u s h r o o m  ", " m u s h r o o m  "),
    (" m u s h\tr o o m  ", " m u s h r o o m  "),
]


@pytest.mark.parametrize("value, expected", REPLACE_WHITESPACE_TESTS)
def test_replaceWhitespace(value, expected):
    result = replaceWhitespace(value)
    assert result == expected


COLLAPSE_WHITESPACE_TESTS = [
    ("\n", ""),
    ("\r", ""),
    ("\t", ""),
    ("\r\t\n", ""),
    ("\t\t \n\n \r\r", ""),
    ("\r \n \t", ""),
    # literal or entity \v and \f are illegal in XML but we shouldn't be touching them
    ("\r\v\t", "\v"),
    ("\r\f\t", "\f"),
    # Python's whitespace (\s, str.strip) definition includes em space (U+2003)
    # but XSD collapse does not so we shouldn't be touching it
    (" \u2003 \u2003  ", "\u2003 \u2003"),
    (" " * 10, ""),
    (" " * 10 + "1", "1"),
    (" " * 10 + "1  1", "1 1"),
    (" " * 10 + "1  1" + " " * 10, "1 1"),
    ("  \r\n  the \tspace  \t\nis  \n  right  \r\n\r\n", "the space is right"),
    (" x  xm   xml    xmln     ", "x xm xml xmln"),
    ("time: \n\tround\n\ttuit  \r\n", "time: round tuit"),
]


@pytest.mark.parametrize("value, expected", COLLAPSE_WHITESPACE_TESTS)
def test_collapseWhitespace(value, expected):
    result = collapseWhitespace(value)
    assert result == expected



from pathlib import Path, PurePath

from tests.integration_tests.validation.conformance_suite_config import (
    AssetSource,
    ConformanceSuiteAssetConfig,
    ConformanceSuiteConfig,
)

config = ConformanceSuiteConfig(
    assets=[
        ConformanceSuiteAssetConfig.conformance_suite(
            Path('table-linkbase-conformance-2024-12-17.zip'),
            entry_point=Path('table-linkbase-conformance-2024-12-17/testcases-index.xml'),
            public_download_url='https://www.xbrl.org/2015/table-linkbase-conformance-2024-12-17.zip',
            source=AssetSource.S3_PUBLIC,
        ),
    ],
    info_url='https://specifications.xbrl.org/work-product-index-table-linkbase-table-linkbase-1.0.html',
    name=PurePath(__file__).stem,
    network_or_cache_required=False,
    shards=4,
    test_case_result_options='match-any',
)



from pathlib import Path, PurePath

from tests.integration_tests.validation.assets import ESEF_PACKAGES, LEI_2020_07_02
from tests.integration_tests.validation.conformance_suite_config import (
    AssetSource,
    ConformanceSuiteAssetConfig,
    ConformanceSuiteConfig,
)

config = ConformanceSuiteConfig(
    args=[
        '--disclosureSystem', 'esef-2024',
    ],
    assets=[
        ConformanceSuiteAssetConfig.conformance_suite(
            Path('esef_conformance_suite_2024.zip'),
            entry_point=Path('esef_conformance_suite_2024/index_inline_xbrl.xml'),
            public_download_url='https://www.esma.europa.eu/sites/default/files/2025-01/esef_conformance_suite_2024.zip',
            source=AssetSource.S3_PUBLIC,
        ),
    ] + [
        package for year in [2017, 2019, 2020, 2021, 2022, 2024] for package in ESEF_PACKAGES[year]
    ],
    expected_additional_testcase_errors={f"esef_conformance_suite_2024/tests/inline_xbrl/{s}": val for s, val in {
        # Typo in the test case namespace declaration: incorrectly uses the Extensible Enumeration 1 namespace with the
        # commonly used Extensible Enumeration 2 prefix: xmlns:enum2="http://xbrl.org/2014/extensible-enumerations"
        'G2-4-1_1/index.xml:TC2_valid': frozenset({'differentExtensionDataType'}),
    }.items()},
    info_url='https://www.esma.europa.eu/document/esef-conformance-suite-2024',
    name=PurePath(__file__).stem,
    network_or_cache_required=False,
    plugins=frozenset({'validate/ESEF'}),
    shards=8,
    test_case_result_options='match-any',
)



from pathlib import Path, PurePath

from tests.integration_tests.validation.conformance_suite_config import (
    AssetSource,
    ConformanceSuiteAssetConfig,
    ConformanceSuiteConfig,
)

config = ConformanceSuiteConfig(
    args=[
        '--calc', 'xbrl21',
    ],
    assets=[
        ConformanceSuiteAssetConfig.conformance_suite(
            Path('XBRL-CONF-2024-12-17.zip'),
            entry_point=Path('XBRL-CONF-2024-12-17/xbrl.xml'),
            public_download_url='https://www.xbrl.org/2014/XBRL-CONF-2024-12-17.zip',
            source=AssetSource.S3_PUBLIC,
        ),
    ],
    expected_additional_testcase_errors={f"XBRL-CONF-2024-12-17/Common/{s}": val for s, val in {
        # 202.02b in the absence of source/target constraints, an empty href doesn't pose a problem
        # 202-02b-HrefResolutionCounterExample-custom.xml Expected: valid, Actual: arelle:hrefWarning
        '200-linkbase/202-xlinkLocator.xml:V-02b': frozenset({'arelle:hrefWarning'}),
    }.items()},
    expected_missing_testcases=frozenset([f"XBRL-CONF-2024-12-17/Common/{s}" for s in [
        "related-standards/xlink/arc-duplication/arc-duplication-testcase.xml",
        "related-standards/xml-schema/uniqueParticleAttribution/uniqueParticleAttribution-testcase.xml",
    ]]),
    info_url='https://specifications.xbrl.org/work-product-index-group-base-spec-base-spec.html',
    name=PurePath(__file__).stem,
    network_or_cache_required=False,
    strict_testcase_index=False,
    shards=3,
)



from __future__ import annotations

import os
import urllib.request
from pathlib import Path

from tests.integration_tests.integration_test_util import get_s3_uri
from tests.integration_tests.scripts.script_util import (
    assert_result,
    parse_args,
    prepare_logfile,
    run_arelle,
    validate_log_file,
)

errors = []
this_file = Path(__file__)
args = parse_args(
    this_file.stem,
    "Confirm unpublished entry point can be loaded from a taxonomy package on the command line.",
)
arelle_command = args.arelle
arelle_offline = args.offline
test_directory = Path(args.test_directory)
arelle_log_file = prepare_logfile(test_directory, this_file)
taxonomy_package_name = "entry_point_from_taxonomy_package.zip"
taxonomy_package_path = test_directory / taxonomy_package_name
taxonomy_package_zip_url = get_s3_uri(
    f"ci/packages/{taxonomy_package_name}", version_id="nbaiNmQvDfvRSZzmrFRsuhH5yS2RDVk."
)

print(f"Downloading taxonomy package: {taxonomy_package_name}")
urllib.request.urlretrieve(taxonomy_package_zip_url, taxonomy_package_path)

entry_point = "https://arelle.org/example/example.xsd"
print(f"Validating entry point: {entry_point}")
run_arelle(
    arelle_command,
    additional_args=[
        "--file",
        entry_point,
        "--packages",
        str(taxonomy_package_path),
        "--validate"
    ],
    offline=arelle_offline,
    logFile=arelle_log_file,
)

errors += validate_log_file(arelle_log_file)

assert_result(errors)

print("Cleaning up")
os.unlink(taxonomy_package_path)
